@@ -128,7 +128,7 @@ where
         _,
     >>::setup(&compound_setup_params)?;
 
-    info!("building merkle tree for the original data");
+    trace!("building merkle tree for the original data");
     let (config, comm_d) = measure_op(Operation::CommD, || -> Result<_> {
         let base_tree_size = get_base_tree_size::<DefaultBinaryTree>(porep_config.sector_size)?;
         let base_tree_leafs = get_base_tree_leafs::<DefaultBinaryTree>(base_tree_size)?;
@@ -166,7 +166,7 @@ where
         Ok((config, comm_d))
     })?;
 
-    info!("verifying pieces");
+    trace!("verifying pieces");
 
     ensure!(
         verify_pieces(&comm_d, piece_infos, porep_config.into())?,
@@ -485,7 +485,7 @@ pub fn seal_commit_phase2<Tree: 'static + MerkleTreeTrait>(
 
     let groth_params = get_stacked_params::<Tree>(porep_config)?;
 
-    info!(
+    trace!(
         "got groth params ({}) while sealing",
         u64::from(PaddedBytesAmount::from(porep_config))
     );
@@ -506,7 +506,7 @@ pub fn seal_commit_phase2<Tree: 'static + MerkleTreeTrait>(
         _,
     >>::setup(&compound_setup_params)?;
 
-    info!("snark_proof:start");
+    trace!("snark_proof:start");
     let groth_proofs = StackedCompound::<Tree, DefaultPieceHasher>::circuit_proofs(
         &public_inputs,
         vanilla_proofs,
@@ -514,7 +514,7 @@ pub fn seal_commit_phase2<Tree: 'static + MerkleTreeTrait>(
         &groth_params,
         compound_public_params.priority,
     )?;
-    info!("snark_proof:finish");
+    trace!("snark_proof:finish");
 
     let proof = MultiProof::new(groth_proofs, &groth_params.pvk);
 
@@ -569,7 +569,7 @@ pub fn get_seal_inputs<Tree: 'static + MerkleTreeTrait>(
     ticket: Ticket,
     seed: Ticket,
 ) -> Result<Vec<Vec<Fr>>> {
-    info!("get_seal_inputs:start");
+    trace!("get_seal_inputs:start");
 
     ensure!(comm_d != [0; 32], "Invalid all zero commitment (comm_d)");
     ensure!(comm_r != [0; 32], "Invalid all zero commitment (comm_r)");
@@ -628,7 +628,7 @@ pub fn get_seal_inputs<Tree: 'static + MerkleTreeTrait>(
         })
         .collect::<Result<_>>()?;
 
-    info!("get_seal_inputs:finish");
+    trace!("get_seal_inputs:finish");
 
     Ok(inputs)
 }
@@ -871,7 +871,7 @@ pub fn verify_aggregate_seal_commit_proofs<Tree: 'static + MerkleTreeTrait>(
         hasher.finalize().into()
     };
 
-    info!("start verifying aggregate proof");
+    trace!("start verifying aggregate proof");
     let result = StackedCompound::<Tree, DefaultPieceHasher>::verify_aggregate_proofs(
         &srs_verifier_key,
         &verifying_key,
@@ -879,7 +879,7 @@ pub fn verify_aggregate_seal_commit_proofs<Tree: 'static + MerkleTreeTrait>(
         commit_inputs.as_slice(),
         &aggregate_proof,
     )?;
-    info!("end verifying aggregate proof");
+    trace!("end verifying aggregate proof");
 
     info!("verify_aggregate_seal_commit_proofs:finish");
 
@@ -893,11 +893,11 @@ pub fn verify_aggregate_seal_commit_proofs<Tree: 'static + MerkleTreeTrait>(
 /// * `porep_config` - this sector's porep config that contains the number of bytes in the sector.
 /// * `piece_infos` - the piece info (commitment and byte length) for each piece in this sector.
 pub fn compute_comm_d(sector_size: SectorSize, piece_infos: &[PieceInfo]) -> Result<Commitment> {
-    info!("compute_comm_d:start");
+    trace!("compute_comm_d:start");
 
     let result = pieces::compute_comm_d(sector_size, piece_infos);
 
-    info!("compute_comm_d:finish");
+    trace!("compute_comm_d:finish");
     result
 }
 
@@ -969,7 +969,7 @@ pub fn verify_seal<Tree: 'static + MerkleTreeTrait>(
         let sector_bytes = PaddedBytesAmount::from(porep_config);
         let verifying_key = get_stacked_verifying_key::<Tree>(porep_config)?;
 
-        info!(
+        trace!(
             "got verifying key ({}) while verifying seal",
             u64::from(sector_bytes)
         );
@@ -1051,7 +1051,7 @@ pub fn verify_batch_seal<Tree: 'static + MerkleTreeTrait>(
     let sector_bytes = PaddedBytesAmount::from(porep_config);
 
     let verifying_key = get_stacked_verifying_key::<Tree>(porep_config)?;
-    info!(
+    trace!(
         "got verifying key ({}) while verifying seal",
         u64::from(sector_bytes)
     );
