@@ -1,34 +1,44 @@
 use std::collections::BTreeMap;
-use std::fs::{read_dir, remove_file};
+use std::fs::{metadata, read_dir, remove_file, OpenOptions};
 use std::io::{Read, Seek, SeekFrom, Write};
 use std::path::{Path, PathBuf};
 use std::sync::Once;
 
-use anyhow::{ensure, Result};
+use anyhow::{ensure, Context, Error, Result};
 use bellperson::groth16;
 use bincode::serialize;
 use blstrs::{Bls12, Scalar as Fr};
 use ff::Field;
 use filecoin_hashers::Hasher;
 use filecoin_proofs::{
-    add_piece, aggregate_seal_commit_proofs, clear_cache, compute_comm_d, fauxrep_aux,
-    generate_fallback_sector_challenges, generate_piece_commitment, generate_single_vanilla_proof,
-    generate_window_post, generate_window_post_with_vanilla, generate_winning_post,
-    generate_winning_post_sector_challenge, generate_winning_post_with_vanilla, get_seal_inputs,
-    seal_commit_phase1, seal_commit_phase2, seal_pre_commit_phase1, seal_pre_commit_phase2,
-    unseal_range, validate_cache_for_commit, validate_cache_for_precommit_phase2,
-    verify_aggregate_seal_commit_proofs, verify_seal, verify_window_post, verify_winning_post,
-    Commitment, DefaultTreeDomain, MerkleTreeTrait, PaddedBytesAmount, PieceInfo, PoRepConfig,
+    add_piece, aggregate_seal_commit_proofs, clear_cache, compute_comm_d, decode_from, encode_into,
+    fauxrep_aux, generate_empty_sector_update_proof,
+    generate_empty_sector_update_proof_with_vanilla, generate_fallback_sector_challenges,
+    generate_partition_proofs, generate_piece_commitment, generate_single_partition_proof,
+    generate_single_vanilla_proof, generate_single_window_post_with_vanilla, generate_window_post,
+    generate_window_post_with_vanilla, generate_winning_post,
+    generate_winning_post_sector_challenge, generate_winning_post_with_vanilla,
+    get_num_partition_for_fallback_post, get_seal_inputs, merge_window_post_partition_proofs,
+    remove_encoded_data, seal_commit_phase1, seal_commit_phase2, seal_pre_commit_phase1,
+    seal_pre_commit_phase2, unseal_range, validate_cache_for_commit,
+    validate_cache_for_precommit_phase2, verify_aggregate_seal_commit_proofs,
+    verify_empty_sector_update_proof, verify_partition_proofs, verify_seal,
+    verify_single_partition_proof, verify_window_post, verify_winning_post, Commitment,
+    DefaultTreeDomain, MerkleTreeTrait, PaddedBytesAmount, PieceInfo, PoRepConfig,
     PoRepProofPartitions, PoStConfig, PoStType, PrivateReplicaInfo, ProverId, PublicReplicaInfo,
     SealCommitOutput, SealPreCommitOutput, SealPreCommitPhase1Output, SectorShape16KiB,
-    SectorShape2KiB, SectorShape32KiB, SectorShape4KiB, SectorSize, UnpaddedByteIndex,
-    UnpaddedBytesAmount, POREP_PARTITIONS, SECTOR_SIZE_16_KIB, SECTOR_SIZE_2_KIB,
-    SECTOR_SIZE_32_KIB, SECTOR_SIZE_4_KIB, WINDOW_POST_CHALLENGE_COUNT, WINDOW_POST_SECTOR_COUNT,
-    WINNING_POST_CHALLENGE_COUNT, WINNING_POST_SECTOR_COUNT,
+    SectorShape2KiB, SectorShape32KiB, SectorShape4KiB, SectorSize, SectorUpdateConfig,
+    UnpaddedByteIndex, UnpaddedBytesAmount, POREP_PARTITIONS, SECTOR_SIZE_16_KIB,
+    SECTOR_SIZE_2_KIB, SECTOR_SIZE_32_KIB, SECTOR_SIZE_4_KIB, WINDOW_POST_CHALLENGE_COUNT,
+    WINDOW_POST_SECTOR_COUNT, WINNING_POST_CHALLENGE_COUNT, WINNING_POST_SECTOR_COUNT,
 };
+use fr32::bytes_into_fr;
+use log::info;
+use memmap::MmapOptions;
 use rand::{random, Rng, SeedableRng};
 use rand_xorshift::XorShiftRng;
 use storage_proofs_core::{api_version::ApiVersion, is_legacy_porep_id, sector::SectorId};
+use storage_proofs_update::constants::TreeRHasher;
 use tempfile::{tempdir, NamedTempFile, TempDir};
 
 // Use a fixed PoRep ID, so that the parents cache can be re-used between some tests.
@@ -73,12 +83,28 @@ fn test_seal_lifecycle_2kib_porep_id_v1_1_base_8() -> Result<()> {
 
 #[test]
 #[ignore]
-fn test_seal_lifecycle_4kib_sub_8_2() -> Result<()> {
+fn test_seal_lifecycle_upgrade_2kib_porep_id_v1_1_base_8() -> Result<()> {
+    let porep_id_v1_1: u64 = 5; // This is a RegisteredSealProof value
+
+    let mut porep_id = [0u8; 32];
+    porep_id[..8].copy_from_slice(&porep_id_v1_1.to_le_bytes());
+    assert!(!is_legacy_porep_id(porep_id));
+    seal_lifecycle_upgrade::<SectorShape2KiB>(SECTOR_SIZE_2_KIB, &porep_id, ApiVersion::V1_1_0)
+}
+
+#[test]
+#[ignore]
+fn test_seal_lifecycle_4kib_sub_8_2_v1() -> Result<()> {
     seal_lifecycle::<SectorShape4KiB>(
         SECTOR_SIZE_4_KIB,
         &ARBITRARY_POREP_ID_V1_0_0,
         ApiVersion::V1_0_0,
-    )?;
+    )
+}
+
+#[test]
+#[ignore]
+fn test_seal_lifecycle_4kib_sub_8_2_v1_1() -> Result<()> {
     seal_lifecycle::<SectorShape4KiB>(
         SECTOR_SIZE_4_KIB,
         &ARBITRARY_POREP_ID_V1_1_0,
@@ -88,12 +114,27 @@ fn test_seal_lifecycle_4kib_sub_8_2() -> Result<()> {
 
 #[test]
 #[ignore]
-fn test_seal_lifecycle_16kib_sub_8_2() -> Result<()> {
+fn test_seal_lifecycle_upgrade_4kib_sub_8_2_v1_1() -> Result<()> {
+    seal_lifecycle_upgrade::<SectorShape4KiB>(
+        SECTOR_SIZE_4_KIB,
+        &ARBITRARY_POREP_ID_V1_1_0,
+        ApiVersion::V1_1_0,
+    )
+}
+
+#[test]
+#[ignore]
+fn test_seal_lifecycle_16kib_sub_8_2_v1() -> Result<()> {
     seal_lifecycle::<SectorShape16KiB>(
         SECTOR_SIZE_16_KIB,
         &ARBITRARY_POREP_ID_V1_0_0,
         ApiVersion::V1_0_0,
-    )?;
+    )
+}
+
+#[test]
+#[ignore]
+fn test_seal_lifecycle_16kib_sub_8_2_v1_1() -> Result<()> {
     seal_lifecycle::<SectorShape16KiB>(
         SECTOR_SIZE_16_KIB,
         &ARBITRARY_POREP_ID_V1_1_0,
@@ -103,12 +144,27 @@ fn test_seal_lifecycle_16kib_sub_8_2() -> Result<()> {
 
 #[test]
 #[ignore]
-fn test_seal_lifecycle_32kib_top_8_8_2() -> Result<()> {
+fn test_seal_lifecycle_upgrade_16kib_sub_8_2_v1_1() -> Result<()> {
+    seal_lifecycle_upgrade::<SectorShape16KiB>(
+        SECTOR_SIZE_16_KIB,
+        &ARBITRARY_POREP_ID_V1_1_0,
+        ApiVersion::V1_1_0,
+    )
+}
+
+#[test]
+#[ignore]
+fn test_seal_lifecycle_32kib_top_8_8_2_v1() -> Result<()> {
     seal_lifecycle::<SectorShape32KiB>(
         SECTOR_SIZE_32_KIB,
         &ARBITRARY_POREP_ID_V1_0_0,
         ApiVersion::V1_0_0,
-    )?;
+    )
+}
+
+#[test]
+#[ignore]
+fn test_seal_lifecycle_32kib_top_8_8_2_v1_1() -> Result<()> {
     seal_lifecycle::<SectorShape32KiB>(
         SECTOR_SIZE_32_KIB,
         &ARBITRARY_POREP_ID_V1_1_0,
@@ -116,11 +172,22 @@ fn test_seal_lifecycle_32kib_top_8_8_2() -> Result<()> {
     )
 }
 
+#[test]
+#[ignore]
+fn test_seal_lifecycle_upgrade_32kib_top_8_8_2_v1_1() -> Result<()> {
+    seal_lifecycle_upgrade::<SectorShape32KiB>(
+        SECTOR_SIZE_32_KIB,
+        &ARBITRARY_POREP_ID_V1_1_0,
+        ApiVersion::V1_1_0,
+    )
+}
+
 // These tests are good to run, but take a long time.
 
 //#[test]
 //#[ignore]
 //fn test_seal_lifecycle_512mib_porep_id_v1_top_8_0_0_api_v1() -> Result<()> {
+//    use filecoin_proofs::{SectorShape512MiB, SECTOR_SIZE_512_MIB};
 //    let porep_id_v1: u64 = 2; // This is a RegisteredSealProof value
 //
 //    let mut porep_id = [0u8; 32];
@@ -132,6 +199,7 @@ fn test_seal_lifecycle_32kib_top_8_8_2() -> Result<()> {
 //#[test]
 //#[ignore]
 //fn test_seal_lifecycle_512mib_porep_id_v1_top_8_0_0_api_v1_1() -> Result<()> {
+//    use filecoin_proofs::{SectorShape512MiB, SECTOR_SIZE_512_MIB};
 //    let porep_id_v1_1: u64 = 7; // This is a RegisteredSealProof value
 //
 //    let mut porep_id = [0u8; 32];
@@ -140,9 +208,21 @@ fn test_seal_lifecycle_32kib_top_8_8_2() -> Result<()> {
 //    seal_lifecycle::<SectorShape512MiB>(SECTOR_SIZE_512_MIB, &porep_id, ApiVersion::V1_1_0)
 //}
 
+//#[test]
+//#[ignore]
+//fn test_seal_lifecycle_upgrade_512mib_top_8_0_0_v1_1() -> Result<()> {
+//    use filecoin_proofs::{SectorShape512MiB, SECTOR_SIZE_512_MIB};
+//    seal_lifecycle_upgrade::<SectorShape512MiB>(
+//        SECTOR_SIZE_512_MIB,
+//        &ARBITRARY_POREP_ID_V1_1_0,
+//        ApiVersion::V1_1_0,
+//    )
+//}
+
 //#[test]
 //#[ignore]
 //fn test_seal_lifecycle_32gib_porep_id_v1_top_8_8_0_api_v1() -> Result<()> {
+//    use filecoin_proofs::{SectorShape32GiB, SECTOR_SIZE_32_GIB};
 //    let porep_id_v1: u64 = 3; // This is a RegisteredSealProof value
 //
 //    let mut porep_id = [0u8; 32];
@@ -154,6 +234,7 @@ fn test_seal_lifecycle_32kib_top_8_8_2() -> Result<()> {
 //#[test]
 //#[ignore]
 //fn test_seal_lifecycle_32gib_porep_id_v1_1_top_8_8_0_api_v1_1() -> Result<()> {
+//    use filecoin_proofs::{SectorShape32GiB, SECTOR_SIZE_32_GIB};
 //    let porep_id_v1_1: u64 = 8; // This is a RegisteredSealProof value
 //
 //    let mut porep_id = [0u8; 32];
@@ -162,9 +243,21 @@ fn test_seal_lifecycle_32kib_top_8_8_2() -> Result<()> {
 //    seal_lifecycle::<SectorShape32GiB>(SECTOR_SIZE_32_GIB, &porep_id, ApiVersion::V1_1_0)
 //}
 
+//#[test]
+//#[ignore]
+//fn test_seal_lifecycle_upgrade_32gib_top_8_8_0_v1_1() -> Result<()> {
+//    use filecoin_proofs::{SectorShape32GiB, SECTOR_SIZE_32_GIB};
+//    seal_lifecycle_upgrade::<SectorShape32GiB>(
+//        SECTOR_SIZE_32_GIB,
+//        &ARBITRARY_POREP_ID_V1_1_0,
+//        ApiVersion::V1_1_0,
+//    )
+//}
+
 //#[test]
 //#[ignore]
 //fn test_seal_lifecycle_64gib_porep_id_v1_top_8_8_2_api_v1() -> Result<()> {
+//    use filecoin_proofs::{SectorShape64GiB, SECTOR_SIZE_64_GIB};
 //    let porep_id_v1: u64 = 4; // This is a RegisteredSealProof value
 //
 //    let mut porep_id = [0u8; 32];
@@ -176,6 +269,7 @@ fn test_seal_lifecycle_32kib_top_8_8_2() -> Result<()> {
 //#[test]
 //#[ignore]
 //fn test_seal_lifecycle_64gib_porep_id_v1_1_top_8_8_2_api_v1_1() -> Result<()> {
+//    use filecoin_proofs::{SectorShape64GiB, SECTOR_SIZE_64_GIB};
 //    let porep_id_v1_1: u64 = 9; // This is a RegisteredSealProof value
 //
 //    let mut porep_id = [0u8; 32];
@@ -184,6 +278,17 @@ fn test_seal_lifecycle_32kib_top_8_8_2() -> Result<()> {
 //    seal_lifecycle::<SectorShape64GiB>(SECTOR_SIZE_64_GIB, &porep_id, ApiVersion::V1_1_0)
 //}
 
+//#[test]
+//#[ignore]
+//fn test_seal_lifecycle_upgrade_64gib_top_8_8_2_v1_1() -> Result<()> {
+//    use filecoin_proofs::{SectorShape64GiB, SECTOR_SIZE_64_GIB};
+//    seal_lifecycle_upgrade::<SectorShape64GiB>(
+//        SECTOR_SIZE_64_GIB,
+//        &ARBITRARY_POREP_ID_V1_1_0,
+//        ApiVersion::V1_1_0,
+//    )
+//}
+
 fn seal_lifecycle<Tree: 'static + MerkleTreeTrait>(
     sector_size: u64,
     porep_id: &[u8; 32],
@@ -194,7 +299,7 @@ fn seal_lifecycle<Tree: 'static + MerkleTreeTrait>(
     let mut prover_id = [0u8; 32];
     prover_id.copy_from_slice(AsRef::<[u8]>::as_ref(&prover_fr));
 
-    create_seal::<_, Tree>(
+    let (_, replica, _, _) = create_seal::<_, Tree>(
         &mut rng,
         sector_size,
         prover_id,
@@ -202,6 +307,30 @@ fn seal_lifecycle<Tree: 'static + MerkleTreeTrait>(
         porep_id,
         api_version,
     )?;
+    replica.close()?;
+
+    Ok(())
+}
+
+fn seal_lifecycle_upgrade<Tree: 'static + MerkleTreeTrait<Hasher = TreeRHasher>>(
+    sector_size: u64,
+    porep_id: &[u8; 32],
+    api_version: ApiVersion,
+) -> Result<()> {
+    let mut rng = &mut XorShiftRng::from_seed(TEST_SEED);
+    let prover_fr: DefaultTreeDomain = Fr::random(&mut rng).into();
+    let mut prover_id = [0u8; 32];
+    prover_id.copy_from_slice(AsRef::<[u8]>::as_ref(&prover_fr));
+
+    let (_, replica, _, _) = create_seal_for_upgrade::<_, Tree>(
+        &mut rng,
+        sector_size,
+        prover_id,
+        porep_id,
+        api_version,
+    )?;
+    replica.close()?;
+
     Ok(())
 }
 
@@ -664,7 +793,7 @@ fn test_winning_post_empty_sector_challenge() -> Result<()> {
     let sector_size = SECTOR_SIZE_2_KIB;
     let api_version = ApiVersion::V1_1_0;
 
-    let (_, _, _, _) = create_seal::<_, SectorShape2KiB>(
+    let (_, replica, _, _) = create_seal::<_, SectorShape2KiB>(
         &mut rng,
         sector_size,
         prover_id,
@@ -694,6 +823,8 @@ fn test_winning_post_empty_sector_challenge() -> Result<()> {
     )
     .is_err());
 
+    replica.close()?;
+
     Ok(())
 }
 
@@ -791,6 +922,8 @@ fn winning_post<Tree: 'static + MerkleTreeTrait>(
         verify_winning_post::<Tree>(&config, &randomness, &pub_replicas[..], prover_id, &proof)?;
     assert!(valid, "proof did not verify");
 
+    replica.close()?;
+
     Ok(())
 }
 
@@ -1074,6 +1207,166 @@ fn test_window_post_single_partition_matching_2kib_base_8() -> Result<()> {
     )
 }
 
+#[test]
+fn test_window_post_partition_matching_2kib_base_8() -> Result<()> {
+    let sector_size = SECTOR_SIZE_2_KIB;
+    let sector_count = *WINDOW_POST_SECTOR_COUNT
+        .read()
+        .expect("WINDOW_POST_SECTOR_COUNT poisoned")
+        .get(&sector_size)
+        .expect("unknown sector size");
+
+    partition_window_post::<SectorShape2KiB>(
+        sector_size,
+        3, // Validate the scenarios of two partition
+        sector_count,
+        false,
+        ApiVersion::V1_0_0,
+    )?;
+    partition_window_post::<SectorShape2KiB>(
+        sector_size,
+        3,
+        sector_count,
+        true,
+        ApiVersion::V1_0_0,
+    )?;
+    partition_window_post::<SectorShape2KiB>(
+        sector_size,
+        3,
+        sector_count,
+        false,
+        ApiVersion::V1_1_0,
+    )?;
+    partition_window_post::<SectorShape2KiB>(sector_size, 3, sector_count, true, ApiVersion::V1_1_0)
+}
+
+fn partition_window_post<Tree: 'static + MerkleTreeTrait>(
+    sector_size: u64,
+    total_sector_count: usize,
+    sector_count: usize,
+    fake: bool,
+    api_version: ApiVersion,
+) -> Result<()> {
+    use anyhow::anyhow;
+
+    let mut rng = XorShiftRng::from_seed(TEST_SEED);
+
+    let mut sectors = Vec::with_capacity(total_sector_count);
+    let mut pub_replicas = BTreeMap::new();
+    let mut priv_replicas = BTreeMap::new();
+
+    let prover_fr: <Tree::Hasher as Hasher>::Domain = Fr::random(&mut rng).into();
+    let mut prover_id = [0u8; 32];
+    prover_id.copy_from_slice(AsRef::<[u8]>::as_ref(&prover_fr));
+
+    let porep_id = match api_version {
+        ApiVersion::V1_0_0 => ARBITRARY_POREP_ID_V1_0_0,
+        ApiVersion::V1_1_0 => ARBITRARY_POREP_ID_V1_1_0,
+    };
+
+    for _ in 0..total_sector_count {
+        let (sector_id, replica, comm_r, cache_dir) = if fake {
+            create_fake_seal::<_, Tree>(&mut rng, sector_size, &porep_id, api_version)?
+        } else {
+            create_seal::<_, Tree>(
+                &mut rng,
+                sector_size,
+                prover_id,
+                true,
+                &porep_id,
+                api_version,
+            )?
+        };
+        priv_replicas.insert(
+            sector_id,
+            PrivateReplicaInfo::new(replica.path().into(), comm_r, cache_dir.path().into())?,
+        );
+        pub_replicas.insert(sector_id, PublicReplicaInfo::new(comm_r)?);
+        sectors.push((sector_id, replica, comm_r, cache_dir, prover_id));
+    }
+    assert_eq!(priv_replicas.len(), total_sector_count);
+    assert_eq!(pub_replicas.len(), total_sector_count);
+    assert_eq!(sectors.len(), total_sector_count);
+
+    let random_fr: <Tree::Hasher as Hasher>::Domain = Fr::random(&mut rng).into();
+    let mut randomness = [0u8; 32];
+    randomness.copy_from_slice(AsRef::<[u8]>::as_ref(&random_fr));
+
+    let config = PoStConfig {
+        sector_size: sector_size.into(),
+        sector_count,
+        challenge_count: WINDOW_POST_CHALLENGE_COUNT,
+        typ: PoStType::Window,
+        priority: false,
+        api_version,
+    };
+
+    let replica_sectors = priv_replicas
+        .iter()
+        .map(|(sector, _replica)| *sector)
+        .collect::<Vec<SectorId>>();
+
+    let challenges = generate_fallback_sector_challenges::<Tree>(
+        &config,
+        &randomness,
+        &replica_sectors,
+        prover_id,
+    )?;
+
+    let num_sectors_per_chunk = config.sector_count;
+    let mut proofs = Vec::new();
+
+    let partitions = get_num_partition_for_fallback_post(&config, replica_sectors.len());
+    for partition_index in 0..partitions {
+        let sector_ids = replica_sectors
+            .chunks(num_sectors_per_chunk)
+            .nth(partition_index)
+            .ok_or_else(|| anyhow!("invalid number of sectors/partition index"))?;
+
+        let mut partition_priv_replicas = BTreeMap::new();
+        for id in sector_ids {
+            let p_sector = match priv_replicas.get(id) {
+                Some(v) => v,
+                _ => {
+                    continue;
+                }
+            };
+
+            partition_priv_replicas.insert(*id, p_sector);
+        }
+
+        let mut vanilla_proofs = Vec::new();
+        for (sector_id, sector) in partition_priv_replicas.iter() {
+            let sector_challenges = &challenges[sector_id];
+            let single_proof = generate_single_vanilla_proof::<Tree>(
+                &config,
+                *sector_id,
+                sector,
+                sector_challenges,
+            )?;
+
+            vanilla_proofs.push(single_proof);
+        }
+
+        let proof = generate_single_window_post_with_vanilla(
+            &config,
+            &randomness,
+            prover_id,
+            vanilla_proofs,
+            partition_index,
+        )?;
+
+        proofs.push(proof);
+    }
+
+    let final_proof = merge_window_post_partition_proofs(proofs)?;
+    let valid =
+        verify_window_post::<Tree>(&config, &randomness, &pub_replicas, prover_id, &final_proof)?;
+    assert!(valid, "proofs did not verify");
+
+    Ok(())
+}
+
 fn window_post<Tree: 'static + MerkleTreeTrait>(
     sector_size: u64,
     total_sector_count: usize,
@@ -1270,7 +1563,7 @@ fn generate_proof<Tree: 'static + MerkleTreeTrait>(
         ticket,
         seed,
         pre_commit_output.clone(),
-        &piece_infos,
+        piece_infos,
     )?;
 
     clear_cache::<Tree>(cache_dir_path)?;
@@ -1340,7 +1633,7 @@ fn unseal<Tree: 'static + MerkleTreeTrait>(
     assert_eq!(contents.len(), 508);
     assert_eq!(&piece_bytes[508..508 + 508], &contents[..]);
 
-    let computed_comm_d = compute_comm_d(config.sector_size, &piece_infos)?;
+    let computed_comm_d = compute_comm_d(config.sector_size, piece_infos)?;
 
     assert_eq!(
         comm_d, computed_comm_d,
@@ -1512,6 +1805,275 @@ fn create_seal_for_aggregation<R: Rng, Tree: 'static + MerkleTreeTrait>(
     )
 }
 
+fn compare_elements(path1: &Path, path2: &Path) -> Result<(), Error> {
+    info!("Comparing elements between {:?} and {:?}", path1, path2);
+    let f_data1 = OpenOptions::new()
+        .read(true)
+        .open(path1)
+        .with_context(|| format!("could not open path={:?}", path1))?;
+    let data1 = unsafe {
+        MmapOptions::new()
+            .map(&f_data1)
+            .with_context(|| format!("could not mmap path={:?}", path1))
+    }?;
+    let f_data2 = OpenOptions::new()
+        .read(true)
+        .open(path2)
+        .with_context(|| format!("could not open path={:?}", path2))?;
+    let data2 = unsafe {
+        MmapOptions::new()
+            .map(&f_data2)
+            .with_context(|| format!("could not mmap path={:?}", path2))
+    }?;
+    let fr_size = std::mem::size_of::<Fr>() as usize;
+    let end = metadata(path1)?.len() as u64;
+    ensure!(
+        metadata(path2)?.len() as u64 == end,
+        "File sizes must match"
+    );
+
+    for i in (0..end).step_by(fr_size) {
+        let index = i as usize;
+        let fr1 = bytes_into_fr(&data1[index..index + fr_size])?;
+        let fr2 = bytes_into_fr(&data2[index..index + fr_size])?;
+        ensure!(fr1 == fr2, "Data mismatch when comparing elements");
+    }
+    info!("Match found for {:?} and {:?}", path1, path2);
+
+    Ok(())
+}
+
+fn create_seal_for_upgrade<R: Rng, Tree: 'static + MerkleTreeTrait<Hasher = TreeRHasher>>(
+    rng: &mut R,
+    sector_size: u64,
+    prover_id: ProverId,
+    porep_id: &[u8; 32],
+    api_version: ApiVersion,
+) -> Result<(SectorId, NamedTempFile, Commitment, TempDir)> {
+    init_logger();
+
+    let (mut piece_file, _piece_bytes) = generate_piece_file(sector_size)?;
+    let sealed_sector_file = NamedTempFile::new()?;
+    let cache_dir = tempdir().expect("failed to create temp dir");
+
+    let porep_config = porep_config(sector_size, *porep_id, api_version);
+    let config = SectorUpdateConfig::from_porep_config(porep_config);
+    let ticket = rng.gen();
+    let sector_id = rng.gen::<u64>().into();
+
+    let (_piece_infos, phase1_output) = run_seal_pre_commit_phase1::<Tree>(
+        porep_config,
+        prover_id,
+        sector_id,
+        ticket,
+        &cache_dir,
+        &mut piece_file,
+        &sealed_sector_file,
+    )?;
+
+    let pre_commit_output = seal_pre_commit_phase2(
+        porep_config,
+        phase1_output,
+        cache_dir.path(),
+        sealed_sector_file.path(),
+    )?;
+    let comm_r = pre_commit_output.comm_r;
+
+    validate_cache_for_commit::<_, _, Tree>(cache_dir.path(), sealed_sector_file.path())?;
+
+    // Upgrade the cc sector here.
+    let new_sealed_sector_file = NamedTempFile::new()?;
+    let new_cache_dir = tempdir().expect("failed to create temp dir");
+
+    // create and generate some random data in staged_data_file.
+    let (mut new_piece_file, _new_piece_bytes) = generate_piece_file(sector_size)?;
+    let number_of_bytes_in_piece =
+        UnpaddedBytesAmount::from(PaddedBytesAmount(porep_config.sector_size.into()));
+
+    let new_piece_info =
+        generate_piece_commitment(new_piece_file.as_file_mut(), number_of_bytes_in_piece)?;
+    new_piece_file.as_file_mut().seek(SeekFrom::Start(0))?;
+
+    let mut new_staged_sector_file = NamedTempFile::new()?;
+    add_piece(
+        &mut new_piece_file,
+        &mut new_staged_sector_file,
+        number_of_bytes_in_piece,
+        &[],
+    )?;
+
+    let new_piece_infos = vec![new_piece_info];
+
+    // FIXME: New replica (new_sealed_sector_file) is currently 0
+    // bytes -- set a length here to ensure proper mmap later.  Lotus
+    // will already be passing in a destination path of the proper
+    // size in the future, so this is a test specific work-around.
+    let new_replica_target_len = metadata(&sealed_sector_file)?.len();
+    let f_sealed_sector = OpenOptions::new()
+        .read(true)
+        .write(true)
+        .create(true)
+        .open(new_sealed_sector_file.path())
+        .with_context(|| format!("could not open path={:?}", new_sealed_sector_file.path()))?;
+    f_sealed_sector.set_len(new_replica_target_len)?;
+
+    let encoded = encode_into::<Tree>(
+        porep_config,
+        new_sealed_sector_file.path(),
+        new_cache_dir.path(),
+        sealed_sector_file.path(),
+        cache_dir.path(),
+        new_staged_sector_file.path(),
+        &new_piece_infos,
+    )?;
+
+    // Generate a single partition proof
+    let partition_proof = generate_single_partition_proof::<Tree>(
+        config,
+        0, // first partition
+        comm_r,
+        encoded.comm_r_new,
+        encoded.comm_d_new,
+        sealed_sector_file.path(), /* sector key file */
+        cache_dir.path(),          /* sector key path needed for p_aux and t_aux */
+        new_sealed_sector_file.path(),
+        new_cache_dir.path(),
+    )?;
+
+    // Verify the single partition proof
+    let proof_is_valid = verify_single_partition_proof::<Tree>(
+        config,
+        0, // first partition
+        partition_proof,
+        comm_r,
+        encoded.comm_r_new,
+        encoded.comm_d_new,
+    )?;
+    ensure!(proof_is_valid, "Partition proof (single) failed to verify");
+
+    // Generate all partition proofs
+    let partition_proofs = generate_partition_proofs::<Tree>(
+        config,
+        comm_r,
+        encoded.comm_r_new,
+        encoded.comm_d_new,
+        sealed_sector_file.path(), /* sector key file */
+        cache_dir.path(),          /* sector key path needed for p_aux and t_aux */
+        new_sealed_sector_file.path(),
+        new_cache_dir.path(),
+    )?;
+
+    // Verify all partition proofs
+    let proofs_are_valid = verify_partition_proofs::<Tree>(
+        config,
+        &partition_proofs,
+        comm_r,
+        encoded.comm_r_new,
+        encoded.comm_d_new,
+    )?;
+    ensure!(proofs_are_valid, "Partition proofs failed to verify");
+
+    let proof = generate_empty_sector_update_proof_with_vanilla::<Tree>(
+        porep_config,
+        partition_proofs,
+        comm_r,
+        encoded.comm_r_new,
+        encoded.comm_d_new,
+    )?;
+    let valid = verify_empty_sector_update_proof::<Tree>(
+        porep_config,
+        &proof.0,
+        comm_r,
+        encoded.comm_r_new,
+        encoded.comm_d_new,
+    )?;
+    ensure!(valid, "Compound proof failed to verify");
+
+    let proof = generate_empty_sector_update_proof::<Tree>(
+        porep_config,
+        comm_r,
+        encoded.comm_r_new,
+        encoded.comm_d_new,
+        sealed_sector_file.path(), /* sector key file */
+        cache_dir.path(),          /* sector key path needed for p_aux and t_aux */
+        new_sealed_sector_file.path(),
+        new_cache_dir.path(),
+    )?;
+    let valid = verify_empty_sector_update_proof::<Tree>(
+        porep_config,
+        &proof.0,
+        comm_r,
+        encoded.comm_r_new,
+        encoded.comm_d_new,
+    )?;
+    ensure!(valid, "Compound proof failed to verify");
+
+    let decoded_sector_file = NamedTempFile::new()?;
+    // FIXME: New replica (new_sealed_sector_file) is currently 0
+    // bytes -- set a length here to ensure proper mmap later.  Lotus
+    // will already be passing in a destination path of the proper
+    // size in the future, so this is a test specific work-around.
+    let decoded_sector_target_len = metadata(&sealed_sector_file)?.len();
+    let f_decoded_sector = OpenOptions::new()
+        .read(true)
+        .write(true)
+        .create(true)
+        .open(decoded_sector_file.path())
+        .with_context(|| format!("could not open path={:?}", decoded_sector_file.path()))?;
+    f_decoded_sector.set_len(decoded_sector_target_len)?;
+
+    decode_from::<Tree>(
+        config,
+        decoded_sector_file.path(),
+        new_sealed_sector_file.path(),
+        sealed_sector_file.path(),
+        cache_dir.path(), /* sector key path needed for p_aux (for comm_c/comm_r_last) */
+        encoded.comm_d_new,
+    )?;
+    // When the data is decoded, it MUST match the original new staged data.
+    compare_elements(decoded_sector_file.path(), new_staged_sector_file.path())?;
+
+    decoded_sector_file.close()?;
+
+    // Remove Data here
+    let remove_encoded_file = NamedTempFile::new()?;
+    let remove_encoded_cache_dir = tempdir().expect("failed to create temp dir");
+    // FIXME: New replica (new_sealed_sector_file) is currently 0
+    // bytes -- set a length here to ensure proper mmap later.  Lotus
+    // will already be passing in a destination path of the proper
+    // size in the future, so this is a test specific work-around.
+    let remove_encoded_target_len = metadata(&sealed_sector_file)?.len();
+    let f_remove_encoded = OpenOptions::new()
+        .read(true)
+        .write(true)
+        .create(true)
+        .open(remove_encoded_file.path())
+        .with_context(|| format!("could not open path={:?}", remove_encoded_file.path()))?;
+    f_remove_encoded.set_len(remove_encoded_target_len)?;
+
+    // Note: we pass cache_dir to the remove, which is the original
+    // dir where the data was sealed.  If this is an API flow problem,
+    // we really just need it to load p_aux for comm_c and could
+    // instead pass comm_c as a parameter instead(?)
+    remove_encoded_data::<Tree>(
+        config,
+        remove_encoded_file.path(),
+        remove_encoded_cache_dir.path(),
+        new_sealed_sector_file.path(),
+        cache_dir.path(),
+        new_staged_sector_file.path(),
+        encoded.comm_d_new,
+    )?;
+    // When the data is removed, it MUST match the original sealed data.
+    compare_elements(remove_encoded_file.path(), sealed_sector_file.path())?;
+
+    remove_encoded_file.close()?;
+
+    clear_cache::<Tree>(cache_dir.path())?;
+
+    Ok((sector_id, sealed_sector_file, comm_r, cache_dir))
+}
+
 fn create_fake_seal<R: rand::Rng, Tree: 'static + MerkleTreeTrait>(
     mut rng: &mut R,
     sector_size: u64,
